/*
  Perdendo uns quilinhos com o Arduino
*/  
  
#include <EEPROM.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>


// Display
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13
Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);
//////////

// INs
const int pinoModoOper     = 6; // 0: Timer; 1: Setup
const int pinoInicio       = 5;
const int ledWork          = 8;
const int ledRest          = 7;
const int pinoAjuste       = A0;

// Outs
const int pinoBuzz         = 3;

// Variaveis globais
int tempoMax               = 40;
int timer1                 = 20;
int timer2                 = 10;
int maxRounds              = 8;
int inicio = 0;

// Rotinas de Inicializacao
void setup() {
      
  // Inicializa Display
  display.begin(SSD1306_SWITCHCAPVCC);
  delay(2000);
  display.clearDisplay();
  display.display();
  
  // Inicializa Pinos
  pinMode(pinoModoOper, INPUT);
  pinMode(pinoInicio, INPUT);
  pinMode(pinoBuzz, OUTPUT);  
  pinMode(ledWork, OUTPUT);  
  pinMode(ledRest, OUTPUT);    
  pinMode(pinoAjuste, INPUT);
  
  //Carrega Padrões
  carregaPadroes();

  // Inicializa os leds indicadores
  digitalWrite(ledWork, LOW);
  digitalWrite(ledRest, LOW);  
  
  // Inicializa a serial para debug
  Serial.begin(9600);
}


// 
void loop() {
  // determina o modo de operação
  int modo = digitalRead(pinoModoOper);

  if (modo == HIGH) {     
    Serial.println(" -- Modo SETUP -- ");
    
    // Setup
    setupTimer();
  } 
  else {
    Serial.println(" -- Modo OPERACAO -- ");
    display.drawPixel(5, 5, WHITE);
    display.display();
    inicio = digitalRead(pinoInicio);
    delay(150);
    display.drawPixel(5, 5, BLACK);
    display.display();    
    delay(150);
    
    // Timer    
    if(inicio)
    {
      Serial.println(" -- INICIO Pressionado -- ");
      telaPreTimer();
      operaTimer();
    }
  }  
}

// Tratamento da tela de setup
void setupTimer() {
    
  int pronto = 0;
  int val = 0;
  int modo = 0;
  int tipoAjuste = 0;
  
  // Carrega valores armazenados de timer1,timer2,maxRounds
  carregaPadroes();
  
  // Tratamento da tela de setup do timer
  while(!pronto) {
    
    modo = digitalRead(pinoModoOper);
    Serial.println();
    Serial.print("Tempo: ");
    Serial.println(modo);
    
    if(modo > 5|| modo < -5) {
      pronto = HIGH;
      modo = HIGH;
    }
    
    // Lê tipo de ajuste
    if(digitalRead(pinoInicio)) {

      if(tipoAjuste <= 4) {
        tipoAjuste++;
      } else {
        tipoAjuste = 0;
      }
      delay(500);
     }
     
     Serial.println(tipoAjuste,DEC);
    
    // Lê valor do ajuste
    val = analogRead(pinoAjuste);
    val = map(val, 0, 1023, 0, tempoMax);
    
    switch (tipoAjuste) {
      case 1:
        // timer 1
        timer1 = val;
        break;
      case 2:
        // timer 2
        timer2 = val;
        break;
      case 3:
        // rounds
        maxRounds = val;
        break;
      case 4:
        pronto = HIGH;
        armazenaPadroes();
        break;
    }
    
      telaSetup(timer1,timer2,maxRounds, tipoAjuste, val);
  }
  
  // Limpa display
  display.clearDisplay();
  display.display();
}

// Operação do Timer
void operaTimer() {
  
  for(int r=1; r<=maxRounds; r++) {
    
    if(digitalRead(pinoInicio)) {
      break;   
    }
    
    Serial.print("-> Round: ");
    Serial.print(r);    
    Serial.println("");
    
    // Timer 1  
    Serial.println(" -- Timer1 -- ");
    
    digitalWrite(ledWork, HIGH);
    digitalWrite(ledRest, LOW);
    
    for(int i=1; i<=timer1; i++) {
      if(digitalRead(pinoInicio)) {
        break;       
      }      
      telaTimer(r, 1, i);
      
      if(i > (timer1-3))
      {
        tone(pinoBuzz, 1000, 300);
        delay(700);
      } else {
        delay(1000);// TODO: Ajustar
      }
    }
    tone(pinoBuzz, 1400, 600);
   
    // Timer 2
    Serial.println(" -- Timer2 -- ");
    
    digitalWrite(ledWork, LOW);
    digitalWrite(ledRest, HIGH);    
    
    for(int i=1; i<=timer2; i++) {
      if(digitalRead(pinoInicio)) {
        break;
      }            
      telaTimer(r, 2, i);
      
      if(i > (timer2-3))
      {
        tone(pinoBuzz, 1000, 300);
        delay(700);
      } else {
        delay(1000);// TODO: Ajustar
      }
    }  
    
    tone(pinoBuzz, 1400, 600);
  }
  delay(100);

  digitalWrite(ledWork, LOW);
  digitalWrite(ledRest, LOW);        
  
  display.clearDisplay();
  display.display();
}

// -- Rotinas auxiliares -------------------------------------------------------------------------------- //
// -- Recupera os valores de configuracao da NVRam
void carregaPadroes() {
  Serial.println(" -- Carrega Padroes -- ");
  timer1 = EEPROM.read(0);
  timer2 = EEPROM.read(1);
  maxRounds = EEPROM.read(3);  
}

// -- Armazena os valores de configuracao da NVRam
void armazenaPadroes() {
  Serial.println(" -- Armazena Padroes -- ");
  EEPROM.write(0,timer1);
  EEPROM.write(1,timer2);
  EEPROM.write(3,maxRounds);    
}

// -- Telas --------------------------------------------------------------------------------------- //
// -- Setup
void telaSetup(int tWork, int tRest, int nRounds, int ajusteAtual, int valor) {
  // Monta a tela de setup
  display.clearDisplay();
  display.drawRect(0, 0, display.width(), display.height(), WHITE);
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(4,4);  
  display.println("- Setup -");
  display.setTextSize(1);

  display.setCursor(4,25);  
  display.print("Work:  ");  
  
  // Valor para WORK   
  if(ajusteAtual == 1) {
    // Ilumina se for o campo em ajuste
    display.setTextColor(BLACK, WHITE);
    // Mostra valor lido do POT
    display.print(valor, DEC);
  }else{
    // Mostra valor anteriormente armazenado na EEPROM
    display.print(tWork, DEC);  
  }
  
  display.setTextColor(WHITE);
  display.setCursor(4,35);  
  display.print("Rest:  ");  
  
  // Valor para REST
  if(ajusteAtual == 2){
    // Ilumina se for o campo em ajuste    
    display.setTextColor(BLACK, WHITE);
    display.print(valor, DEC);
  }else{
    // Mostra valor anteriormente armazenado na EEPROM    
    display.print(tRest, DEC);  
  }
  
  display.setTextColor(WHITE);  
  display.setCursor(4,45);  
  display.print("Rounds: ");  
  
  // Valor para ROUNDS
  if(ajusteAtual == 3){
    // Ilumina se for o campo em ajuste      
    display.setTextColor(BLACK, WHITE);
    display.print(valor, DEC);
  }else{
    // Mostra valor anteriormente armazenado na EEPROM
    display.print(maxRounds, DEC);  
  }
  display.setTextColor(WHITE);
    
  display.display();
}

// -- Tela preliminar ao Timer
void telaPreTimer() {
  display.clearDisplay();
  display.setTextSize(4);
  display.setTextColor(WHITE);

  // Contagem regressiva de 3 segundos antes do primeiro ciclo work
  for(int i=3; i>0; i--){
    tone(pinoBuzz, 1000, 300);
    display.setCursor(50,20);  
    display.clearDisplay();
    display.print(i, DEC);            
    display.display();
    delay(700);
  }

  tone(pinoBuzz, 1600, 600);  
  
  display.clearDisplay();
  display.setCursor(30,20);
  display.print("GO!");
  display.display();
  delay(300);
}

// -- Tela de operacao do timer
void telaTimer(int r, int timer, int valTimer) {
  display.clearDisplay();
  display.drawRect(0, 0, display.width(), display.height(), WHITE);
  display.setTextSize(3);
  display.setTextColor(WHITE);
  display.setCursor(4,4);  
  display.setTextSize(2);
  
  if(timer == 1) {
    display.print("WORK: ");
  } else {
    display.print("REST: ");    
  }
  
  display.setTextSize(3);
  display.print(valTimer, DEC);
  display.setCursor(4,34);    
  display.setTextSize(2);
  display.print(r, DEC);
  display.print(" / ");
  display.print(maxRounds);
  display.display();
}


